// ReflectiveFloor_ReflectionMap effect
// 
// Copyright Outerspace Software, all rights reserved

string description = "This effect features: phong shading, colourmapping, reflectionmapping, reflective floor and texture animation. The 1st texture must be a colourmap, the 2nd a reflectionmap. You do not have to use both texture slots.";

float4x4	World;
float4x4	ViewProjection;
float4x4	WorldInverseTranspose;
float4		LightPos;
float4		EyePos;

float4		AmbientCol;
float4		DiffuseCol;
float4		SpecularCol;
float		Power = 100;

float4		TextureResolution0;
float4		TextureResolution1;
float4		UVBounds0;
float4 		UVBounds1;

float		Time;

const float cSmoothFloorClip=10;

float4 Prop0
<
string UIName="Floor Colour";
float UIMin=0;
float UIMax=1;
float3 UIDefault=float3(1,1,1);
int UISliders=3;
float UIScale=255;
bool UIInteger=true;
int UIWidget=2;
> = float4(1,1,1,1);

float4 Prop1
<
string UIName="Transparency Reflection / Blur";
float UIMin=0;
float UIMax=1;
float3 UIDefault=float3(0.5,0,0);
int UISliders=2;
float UIScale=1;
bool UIInteger=false;
int UIWidget=0;
> = float4(0.5,0,0,0);

float4 Prop2
<
string UIName="Transparency Gradient";
float UIMin=0;
float UIMax=0.1;
float3 UIDefault=float3(0.01,0,0);
int UISliders=1;
float UIScale=100;
bool UIInteger=false;
int UIWidget=0;
> = float4(0.01,0,0,0);

float4 Prop3
<
string UIName="Floor Level";
float UIMin=-100;
float UIMax=100;
float3 UIDefault=float3(0,0,0);
int UISliders=1;
float UIScale=1;
bool UIInteger=false;
int UIWidget=0;
> = float4(0,0,0,0);

float4 Prop4
<
string UIName="Reflection Rotation";
float UIMin=-3.14159;
float UIMax=3.14159;
float3 UIDefault=float3(0,0,0);
int UISliders=3;
float UIScale=57.29577;
bool UIInteger=false;
int UIWidget=0;
> = float4(0,0,0,0);

float4 Prop5
<
string UIName="Lighting Factor";
float UIMin=0;
float UIMax=1;
float3 UIDefault=float3(1,0,0);
int UISliders=1;
float UIScale=1;
bool UIInteger=false;
int UIWidget=0;
> = float4(1,0,0,0);

float4 Prop6
<
string UIName="Reflection Factor";
float UIMin=0;
float UIMax=1;
float3 UIDefault=float3(1,0,0);
int UISliders=1;
float UIScale=1;
bool UIInteger=false;
int UIWidget=0;
> = float4(1,0,0,0);

float4 Prop7
<
string UIName="Texture Speed";
float UIMin=-1;
float UIMax=1;
float3 UIDefault=float3(0,0,0);
int UISliders=2;
float UIScale=1;
bool UIInteger=false;
int UIWidget=0;
> = float4(0,0,0,0);

texture MyTexture0;
sampler MyTextureSampler0 = sampler_state
{
   Texture = <MyTexture0>;
   MinFilter = Linear;
   MagFilter = Linear;
   MipFilter = Linear;   
   AddressU  = Wrap;
   AddressV  = Wrap;
};

texture MyTexture1;
sampler MyTextureSampler1 = sampler_state
{
   Texture = <MyTexture1>;
   MinFilter = Linear;
   MagFilter = Linear;
   MipFilter = Linear;   
   AddressU  = Wrap;
   AddressV  = Wrap;
};

struct VS_INPUT{
	float4 position:	POSITION;
	float3 normal:		NORMAL;
	float3 diffuse:		COLOR;
	float2 tex:		TEXCOORD0;
};

struct VS_OUTPUT{
	float4 position:	POSITION;
	float2 tex:		TEXCOORD0;
	float4 worldpos:	TEXCOORD1;
	float4 worldposR:	TEXCOORD2;
	float3 normal:		TEXCOORD3;	
	float2 tex_colourmap:	TEXCOORD4;	
};

struct PS_OUTPUT{
    float4 color:		COLOR;
};

float4 GetAntialiasedPixel(bool inMirror, int inSampler, float2 c){

	float b=2*Prop1.y;

	if(!inMirror || b<=0){
		if(inSampler==0) return tex2D(MyTextureSampler0,c);
		if(inSampler==1) return tex2D(MyTextureSampler1,c);
		return float4(1,1,1,1);
	}

	float dx=1;
	float dy=1;

	if(inSampler==0){
		dx=b/TextureResolution0.x;
		dy=b/TextureResolution0.y;
	}
	if(inSampler==1){
		dx=b/TextureResolution1.x;
		dy=b/TextureResolution1.y;
	}

	float4 p=float4(0,0,0,0);

	const int AA=4;

	c-=float2(AA*0.5*dx,AA*0.5*dy);

	for(int i=0; i<AA; i++){
		for(int j=0; j<AA; j++){


			float2 c2=c+float2(dx*i,dy*j);

			if(inSampler==0) p+=tex2D(MyTextureSampler0,c2);
			if(inSampler==1) p+=tex2D(MyTextureSampler1,c2);
		}
	}

	p/=(AA*AA);

	return p;
}

float3 RotateReflection(float3 a){
	if(Prop4.x==0 && Prop4.y==0 && Prop4.z==0) return a;

	float sa=sin(Prop4.x);
	float ca=cos(Prop4.x);
	float sb=sin(Prop4.y);
	float cb=cos(Prop4.y);
	float sg=sin(Prop4.z);
	float cg=cos(Prop4.z);

	float3 b;

	b.x=a.x*ca-a.z*sa;
	b.y=a.y;
	b.z=a.x*sa+a.z*ca;

	a.x=b.x;
	a.y=b.y*cb-b.z*sb;
	a.z=b.y*sb+b.z*cb;

	b.x=a.x*cg-a.y*sg;
	b.y=a.x*sg+a.y*cg;
	b.z=a.z;

	return b;
}

float2 Normal_2_UVBounds0(float2 inUV){
	inUV.x = UVBounds0.x + inUV.x*(UVBounds0.y-UVBounds0.x);
	inUV.y = UVBounds0.z + inUV.y*(UVBounds0.w-UVBounds0.z);
	return inUV;
}

float2 UVBOunds0_2_Normal(float2 inUV){
	inUV.x=(inUV.x-UVBounds0.x)/(UVBounds0.y-UVBounds0.x);
	inUV.y=(inUV.y-UVBounds0.z)/(UVBounds0.w-UVBounds0.z);
	return inUV;
}

float2 Normal_2_UVBounds1(float2 inUV){
	inUV.x = UVBounds1.x + inUV.x*(UVBounds1.y-UVBounds1.x);
	inUV.y = UVBounds1.z + inUV.y*(UVBounds1.w-UVBounds1.z);
	return inUV;
}

//spiegelbeeld
VS_OUTPUT VS1(VS_INPUT input){
	VS_OUTPUT output;  
	
	output.tex_colourmap=Normal_2_UVBounds0(UVBOunds0_2_Normal(input.tex)+Prop7.xy*Time);
	
	float4 pos  = mul(input.position, World);

	float4 posR = pos;
	posR.y = 2*Prop3.x - pos.y;		
	
	output.worldpos 	= pos;
	output.worldposR 	= posR;
	output.position 	= mul(posR, ViewProjection);

	float3 NormalVector	= normalize(mul(input.normal,WorldInverseTranspose));
	float3 EyeVector	= normalize(EyePos - pos);



	float3 ReflectionVector = reflect(EyeVector,NormalVector);

	ReflectionVector=RotateReflection(ReflectionVector);

	ReflectionVector.z=abs(ReflectionVector.z);

	ReflectionVector.z+=1;
	float f=2*length(ReflectionVector);
	output.tex = float2(0.5,0.5) + ReflectionVector.xy/f;

	output.tex=Normal_2_UVBounds1(output.tex);




	output.normal		= NormalVector;
	return output;
}

//normaal beeld
VS_OUTPUT VS2(VS_INPUT input){
	VS_OUTPUT output;      

	output.tex_colourmap=Normal_2_UVBounds0(UVBOunds0_2_Normal(input.tex)+Prop7.xy*Time);

	float4 pos = mul(input.position, World);

	output.worldpos 	= pos;
	output.worldposR 	= pos;
	output.position 	= mul(pos, ViewProjection);

	float3 NormalVector	= normalize(mul(input.normal,WorldInverseTranspose));
	float3 EyeVector	= normalize(EyePos - pos);




	float3 ReflectionVector = reflect(EyeVector,NormalVector);

	ReflectionVector=RotateReflection(ReflectionVector);

	ReflectionVector.z=abs(ReflectionVector.z);

	ReflectionVector.z+=1;
	float f=2*length(ReflectionVector);
	output.tex = float2(0.5,0.5) + ReflectionVector.xy/f;

	output.tex=Normal_2_UVBounds1(output.tex);




	output.normal		= NormalVector;
	return output;
}

float4 DoIt(VS_OUTPUT input, bool inMirror){
	float3 NormalVector		= normalize(input.normal);
	float3 LightVector		= normalize(LightPos - input.worldpos);
	float3 EyeVector		= normalize(EyePos   - input.worldpos);
	float3 LightReflectionVector	= -reflect(LightVector,NormalVector);

	//ambient component
	float4 c=AmbientCol;

	//diffuse component
	c += DiffuseCol * saturate(dot(NormalVector, LightVector));

	//specular component
	if(Power<=0) c += SpecularCol;
	if(Power>0)  c += SpecularCol * pow(saturate(dot(EyeVector, LightReflectionVector)),Power);
	
	c.w = 1;

	c=DiffuseCol*(1-Prop5.x) + c*Prop5.x;

	if(TextureResolution0.x>0){
		float4 d=GetAntialiasedPixel(inMirror,0,input.tex_colourmap);
		c*=d;
	}

	if(TextureResolution1.x>0){
		float4 d=GetAntialiasedPixel(inMirror,1,input.tex);

		d=(d*Prop6.x)+(float4(1,1,1,1)*(1-Prop6.x));

		c*=d;


	}

	return c;
}


//spiegelbeeld
PS_OUTPUT PS1(VS_OUTPUT input){
	PS_OUTPUT output;

	float4 c=DoIt(input,true);

	if(input.worldposR.y > Prop3.x){
		c.w*=(1-(input.worldposR.y-Prop3.x)*cSmoothFloorClip);
		if(c.w<0) c.w=0;
	}

	c.w*=(1-Prop1.x);

	float f=1-(-(input.worldposR.y-Prop3.x) * Prop2.x);
	if(f<0) f=0;
	if(f>1) f=1;
	c.w*=f;
	c.w*=DiffuseCol.w;

	c.xyz *= Prop0.xyz;

	output.color=c;
	return output;
}

//normaal beeld
PS_OUTPUT PS2(VS_OUTPUT input){
	PS_OUTPUT output;

	float4 c=DoIt(input,false);

	if(input.worldpos.y < Prop3.x){
		c.w*=(1-(Prop3.x-input.worldposR.y)*cSmoothFloorClip);
		if(c.w<0) c.w=0;
	}

	c.w*=DiffuseCol.w;
	
	output.color = c;
	return output;
}

technique Shader
{
    //mirror: alleen z-buffer schrijven
    pass P1
    {
	Alphablendenable= TRUE;
	SRCBLEND	= SRCALPHA;
	DESTBLEND	= INVSRCALPHA;
	ALPHAARG1[0]	= DIFFUSE;
	ALPHAOP[0]	= SELECTARG1;
	ALPHATESTENABLE	= TRUE;
	ALPHAREF	= 1;
	ALPHAFUNC	= GREATEREQUAL;
	CullMode	= CW;
	ZFUNC		= LESSEQUAL;

	ColorWriteEnable=0;

        VertexShader	= compile vs_3_0 VS1();
        PixelShader	= compile ps_3_0 PS1();
    }

    //mirror: RGB
    pass P0
    {
	ColorWriteEnable=7;

        VertexShader	= compile vs_3_0 VS1();
        PixelShader	= compile ps_3_0 PS1();
    }

    //normaal    
    pass P1
    {
	CullMode	= CCW;
        VertexShader	= compile vs_3_0 VS2();
        PixelShader	= compile ps_3_0 PS2();
    }
}

technique AlphaShader
{
    //mirror: alleen z-buffer schrijven
    pass P1
    {
	Alphablendenable= TRUE;
	SRCBLEND	= SRCALPHA;
	DESTBLEND	= INVSRCALPHA;
	ALPHAARG1[0]	= DIFFUSE;
	ALPHAOP[0]	= SELECTARG1;
	ALPHATESTENABLE	= TRUE;
	ALPHAREF	= 1;
	ALPHAFUNC	= GREATEREQUAL;
	CullMode	= CW;
	ZFUNC		= LESSEQUAL;

	ColorWriteEnable=0;

        VertexShader	= compile vs_3_0 VS1();
        PixelShader	= compile ps_3_0 PS1();
    }

    //mirror: RGB
    pass P0
    {
	ColorWriteEnable=8;
		
        VertexShader	= compile vs_3_0 VS1();
        PixelShader	= compile ps_3_0 PS1();
    }

    //normaal    
    pass P1
    {
	CullMode	= CCW;
        VertexShader	= compile vs_3_0 VS2();
        PixelShader	= compile ps_3_0 PS2();
    }
}

