// CubeMapDDS
//
// Copyright Outerspace Software, all rights reserved

string description = "A reflection mapping shader using the DirectX DDS cubemap picture format. The texture should be a DDS cubemap. The second texture is not used.";

float4x4	World;
float4x4	ViewProjection;
float4x4	WorldInverseTranspose;
float4		EyePos;
float4		DiffuseCol;

float4 Prop0
<
string UIName="Reflection Rotation";
float UIMin=-3.14159;
float UIMax=3.14159;
float3 UIDefault=float3(0,0,0);
int UISliders=3;
float UIScale=57.29577;
bool UIInteger=false;
int UIWidget=0;
> = float4(0,0,0,0);

texture MyTexture0;
samplerCUBE MyTextureSampler0 = sampler_state {
	Texture = <MyTexture0>;
	MinFilter = Linear;
	MagFilter = Linear;
	MipFilter = Linear;
};

struct VS_INPUT{
	float4 position:	POSITION;
	float3 normal:		NORMAL;
	float3 diffuse:		COLOR;
	float2 tex:		TEXCOORD0;
};

struct VS_OUTPUT{
	float4 position:	POSITION;
	float3 worldpos:	TEXCOORD0;
	float3 normal:		TEXCOORD1;
};

struct PS_OUTPUT{
    float4 color:		COLOR;
};

float3 RotateReflection(float3 a){
	if(Prop0.x==0 && Prop0.y==0 && Prop0.z==0) return a;

	float sa=sin(Prop0.x);
	float ca=cos(Prop0.x);
	float sb=sin(Prop0.y);
	float cb=cos(Prop0.y);
	float sg=sin(Prop0.z);
	float cg=cos(Prop0.z);

	float3 b;

	b.x=a.x*ca-a.z*sa;
	b.y=a.y;
	b.z=a.x*sa+a.z*ca;

	a.x=b.x;
	a.y=b.y*cb-b.z*sb;
	a.z=b.y*sb+b.z*cb;

	b.x=a.x*cg-a.y*sg;
	b.y=a.x*sg+a.y*cg;
	b.z=a.z;

	return b;
}

VS_OUTPUT VS(VS_INPUT input){
	VS_OUTPUT output;

	float4 Pos		= mul(input.position, World);
	output.position		= mul(Pos, ViewProjection);

	output.worldpos = Pos;
	output.normal   = mul(input.normal,WorldInverseTranspose);

	return output;
}

PS_OUTPUT PS(VS_OUTPUT input){
	PS_OUTPUT output;

	float3 NormalVector	= normalize(input.normal);
	float3 EyeVector	= normalize(EyePos-input.worldpos);
	float3 ReflectionVector	= normalize(RotateReflection(reflect(NormalVector,EyeVector)));
  
    	output.color.xyz = DiffuseCol * texCUBE(MyTextureSampler0,ReflectionVector);
    	output.color.w = DiffuseCol.w;
	return output;
}

technique Shader
{
    pass P0
    {
        VertexShader = compile vs_1_1 VS();
        PixelShader  = compile ps_2_0 PS();
    }
}

