// ColourMap_CubeMapDDS effect
// 
// Copyright Outerspace Software, all rights reserved

string description = "This effect features colourmapping and cubemapping. The 1st texture must be a colourmap, the 2nd a DDS cubemap. You do not have to use both texture slots.";

float4x4	World;
float4x4	ViewProjection;
float4x4	WorldInverseTranspose;
float4		LightPos;
float4		EyePos;

float4		AmbientCol;
float4		DiffuseCol;
float4		SpecularCol;
float		Power = 100;

float4		TextureResolution0;
float4		TextureResolution1;
float4		UVBounds0;
float4 		UVBounds1;

float4 Prop0
<
string UIName="Lighting Factor";
float UIMin=0;
float UIMax=1;
float3 UIDefault=float3(1,0,0);
int UISliders=1;
float UIScale=1;
bool UIInteger=false;
int UIWidget=0;
> = float4(1,0,0,0);

float4 Prop1
<
string UIName="Reflection Factor";
float UIMin=0;
float UIMax=1;
float3 UIDefault=float3(1,0,0);
int UISliders=1;
float UIScale=1;
bool UIInteger=false;
int UIWidget=0;
> = float4(1,0,0,0);

float4 Prop2
<
string UIName="Reflection Rotation";
float UIMin=-3.14159;
float UIMax=3.14159;
float3 UIDefault=float3(0,0,0);
int UISliders=3;
float UIScale=57.29577;
bool UIInteger=false;
int UIWidget=0;
> = float4(0,0,0,0);

texture MyTexture0;
sampler MyTextureSampler0 = sampler_state
{
   Texture = <MyTexture0>;
   MinFilter = Linear;
   MagFilter = Linear;
   MipFilter = Linear;   
   AddressU  = Wrap;
   AddressV  = Wrap;
};

texture MyTexture1;
samplerCUBE MyTextureSampler1 = sampler_state {
	Texture = <MyTexture1>;
	MinFilter = Linear;
	MagFilter = Linear;
	MipFilter = Linear;
};

struct VS_INPUT{
	float4 position:	POSITION;
	float3 normal:		NORMAL;
	float3 diffuse:		COLOR;
	float2 tex:		TEXCOORD0;
};

struct VS_OUTPUT{
	float4 position:	POSITION;
	float4 worldpos:	TEXCOORD0;
	float3 normal:		TEXCOORD1;	
	float2 tex_colourmap:	TEXCOORD2;	
};

struct PS_OUTPUT{
    float4 color:		COLOR;
};

float3 RotateReflection(float3 a){
	if(Prop2.x==0 && Prop2.y==0 && Prop2.z==0) return a;

	float sa=sin(Prop2.x);
	float ca=cos(Prop2.x);
	float sb=sin(Prop2.y);
	float cb=cos(Prop2.y);
	float sg=sin(Prop2.z);
	float cg=cos(Prop2.z);

	float3 b;

	b.x=a.x*ca-a.z*sa;
	b.y=a.y;
	b.z=a.x*sa+a.z*ca;

	a.x=b.x;
	a.y=b.y*cb-b.z*sb;
	a.z=b.y*sb+b.z*cb;

	b.x=a.x*cg-a.y*sg;
	b.y=a.x*sg+a.y*cg;
	b.z=a.z;

	return b;
}

float2 Normal_2_UVBounds0(float2 inUV){
	inUV.x = UVBounds0.x + inUV.x*(UVBounds0.y-UVBounds0.x);
	inUV.y = UVBounds0.z + inUV.y*(UVBounds0.w-UVBounds0.z);
	return inUV;
}

float2 UVBounds0_2_Normal(float2 inUV){
	inUV.x=(inUV.x-UVBounds0.x)/(UVBounds0.y-UVBounds0.x);
	inUV.y=(inUV.y-UVBounds0.z)/(UVBounds0.w-UVBounds0.z);
	return inUV;
}

VS_OUTPUT VS(VS_INPUT input){
	VS_OUTPUT output;      

	output.tex_colourmap=Normal_2_UVBounds0(UVBounds0_2_Normal(input.tex));

	float4 pos = mul(input.position, World);

	output.worldpos 	= pos;
	output.position 	= mul(pos, ViewProjection);
	output.normal		= mul(input.normal,WorldInverseTranspose);
	return output;
}

PS_OUTPUT PS(VS_OUTPUT input){
	PS_OUTPUT output;

	float3 NormalVector		= normalize(input.normal);
	float3 LightVector		= normalize(LightPos - input.worldpos);
	float3 EyeVector		= normalize(EyePos   - input.worldpos);
	float3 LightReflectionVector	= -reflect(LightVector,NormalVector);

	//ambient component
	float4 c=AmbientCol;

	//diffuse component
	c += DiffuseCol * saturate(dot(NormalVector, LightVector));

	//specular component
	if(Power<=0) c += SpecularCol;
	if(Power>0)  c += SpecularCol * pow(saturate(dot(EyeVector, LightReflectionVector)),Power);
	
	c.w = 1;

	c=DiffuseCol*(1-Prop0.x) + c*Prop0.x;

	if(TextureResolution0.x>0){
		float4 d=tex2D(MyTextureSampler0,input.tex_colourmap);
		c*=d;
	}

	if(TextureResolution1.x>0){

		float3 ReflectionVector	= normalize(RotateReflection(reflect(NormalVector,EyeVector)));

		float4 d=texCUBE(MyTextureSampler1,ReflectionVector);

		d=(d*Prop1.x)+(float4(1,1,1,1)*(1-Prop1.x));

		c*=d;
	}

	c.w*=DiffuseCol.w;
	
	output.color = c;
	return output;
}

technique Shader
{
    pass P1
    {
        VertexShader	= compile vs_2_0 VS();
        PixelShader	= compile ps_2_0 PS();
    }
}
